#ifndef NVIDIA_LIB_H__
#define NVIDIA_LIB_H__

#include <stdio.h>
#include "settings.h"

extern int __new_glx_scheme_detect_result;

struct nvver
{
    long num[VER_NUMBERS];
};

struct xinflist
{
    struct xinflist *next;
    char xinf[PATH_MAX];
    char module[PATH_MAX];
    char version[VER_LEN_MAX];
    unsigned int supported;
};

struct nvdevlist
{
    struct nvdevlist *next;
    unsigned int vendor;
    unsigned int device;
    struct xinflist *xinf;
};

struct envglobal
{
    unsigned int debug_level;
    int o_verbose;
    int other_videocards;
    int intel_videocards;
    int nvidia_videocards;
    int nvidia_3dcontrollers;
    char nv_drv_dir[PATH_MAX];
    char nv_drv32_dir[PATH_MAX];
    char last_kmod_ver_str[VER_LEN_MAX];
    char new_kmod_ver_str[VER_LEN_MAX];
    char proc_kmod_ver_str[VER_LEN_MAX];
    char kernel_ver_str[32];
};

struct nvdevlist* new_nvidia_dev(struct nvdevlist **cards, unsigned int vnd, unsigned int dev, struct xinflist *xnf);
int mk_symlink_from_to(int verbosity, const char *from_file, const char *from_dir, const char *to_file, const char *to_dir);
int mk_symlink_to(int verbosity, const char *from_file, const char *to_dir, const char *to_file);
int mk_symlink32_to(int verbosity, const char *from_file, const char *to_dir, const char *to_file);
int mk_symlink(int verbosity, const char *to_file, const char *to_dir);
int mk_symlink32(int verbosity, const char *to_file, const char *to_dir);
void unlink_file(const char *dir, const char *file);
void restore_symlink(int verbosity, const char *to_file, const char *from_dir, const char *to_dir);
void get_nv_drv_dir();
void restore_symlinks();
int read_version_from_file(char *version, const char *path);
int read_version_from_proc(char *version, const char *path);
int write_version_to_file(const char *version, const char *path);
void init_global();
void get_kernel_ver();
void get_last_kmod_ver();
int version_compare(const struct nvver ver1, const struct nvver ver2);
struct nvver string_to_version(const char *version);
int detected_libs_32();
int is_nvidia_plain_symlink(const char *path);
int is_exists_dir_file(const char *dir,const char *file);
int using_glvnd_scheme();

#endif
