#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <limits.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>

#include "settings.h"
#include "lib.h"

extern const char *__progname;
struct envglobal g;

void program_version( FILE *output )
{
    fprintf(output, PACKAGE" X11 driver switcher v%s by Sergey Turchin <zerg@altlinux.org>\n", PACKAGE_VERSION);
}

void program_usage( FILE *output )
{
    program_version( output );
    fprintf(output,
"Usage:\n"
"  nvidia -h				-- display command usage\n"
"  nvidia -V				-- display program version\n"
"  nvidia -v, --verbose			-- enable verbose output\n"
);
}

int main(int argc, char **argv)
{
    static struct option long_options[] = { 
	{ "verbose", 0, NULL, 'v' }, 
	{ "help", 0, NULL, 'h' }, 
	{ "version", 0, NULL, 'V'},
	{ NULL, 0, NULL, 0 } };

    init_global();

    while(1)
    {
	int c = getopt_long( argc, argv, "hvV", long_options, NULL );
	if( c == -1 ) 
    	    break;
	switch( c )
	{
	    case 'v':
		g.o_verbose = 1;
		break;
	    case 'V':
		program_version( stdout );
		exit(0);
	    case 'h':
		program_usage( stdout );
		exit(0);
	    default: 
		program_usage( stderr );
		exit(1);
	}
    }

    get_kernel_ver();
    get_last_kmod_ver();
    get_nv_drv_dir();

    restore_symlinks();

    /* make symlinks */
    /* restore symlinks from other packages */
    if( using_glvnd_scheme() == 0 ) {
        mk_symlink_from_to(0, "libdri.so", X_DRV_DIR, "libdri.so", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libglx.so") )
	    mk_symlink(0, "libglx.so", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libglx.so", X_DRV_DIR, "libglx.so", X_ETCLIB_SYML_DIR);
        mk_symlink(0, "libGL.so.1", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libGLESv2.so.2") )
	    mk_symlink(0, "libGLESv2.so.2", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libGL.so.1", g.nv_drv_dir, "libGLESv2.so.2", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libGLESv1_CM.so.1") )
	    mk_symlink(0, "libGLESv1_CM.so.1", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libGL.so.1", g.nv_drv_dir, "libGLESv1_CM.so.1", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libEGL.so.1") )
	    mk_symlink(0, "libEGL.so.1", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libEGL.so.1", X_DRV_DIR, "libEGL.so.1", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libGLdispatch.so.0") )
	    mk_symlink(0, "libGLdispatch.so.0", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libGLdispatch.so.0", X_DRV_DIR, "libGLdispatch.so.0", X_ETCLIB_SYML_DIR);
        if( is_exists_dir_file(g.nv_drv_dir, "libGLX.so.0") )
	    mk_symlink(0, "libGLX.so.0", X_ETCLIB_SYML_DIR);
	else
	    mk_symlink_from_to(0, "libGLX.so.0", X_DRV_DIR, "libGLX.so.0", X_ETCLIB_SYML_DIR);
        mk_symlink(-1, "libwfb.so", X_ETCLIB_SYML_DIR);
    }

    /* restore symlinks from nvidia packages */
    mk_symlink(0, "nvidia_drv.so", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libGLcore.so.1", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libnvidia-cfg.so.1", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libvdpau_nvidia.so", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libEGL_nvidia.so.0", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libGLESv2_nvidia.so.2", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libGLESv1_CM_nvidia.so.1", NV_ETCLIB_SYML_DIR);
    mk_symlink(-1, "libGLX_nvidia.so.0", NV_ETCLIB_SYML_DIR);

#if defined(__x86_64__) || defined(__aarch64__)
	if( detected_libs_32() )
	{
	    /* restore symlinks from other packages */
	    if( using_glvnd_scheme() == 0 ) {
		mk_symlink32(0, "libGL.so.1", X_ETCLIB_SYML32_DIR);
		if( is_exists_dir_file(g.nv_drv32_dir, "libGLESv2.so.2") )
		    mk_symlink32(0, "libGLESv2.so.2", X_ETCLIB_SYML32_DIR);
		else
		    mk_symlink_from_to(0, "libGL.so.1", g.nv_drv32_dir, "libGLESv2.so.2", X_ETCLIB_SYML32_DIR);
		if( is_exists_dir_file(g.nv_drv32_dir, "libGLESv1_CM.so.1") )
		    mk_symlink32(0, "libGLESv1_CM.so.1", X_ETCLIB_SYML32_DIR);
		else
		    mk_symlink_from_to(0, "libGL.so.1", g.nv_drv32_dir, "libGLESv1_CM.so.1", X_ETCLIB_SYML32_DIR);
		if( is_exists_dir_file(g.nv_drv32_dir, "libEGL.so.1") )
		    mk_symlink32(0, "libEGL.so.1", X_ETCLIB_SYML32_DIR);
		else
		    mk_symlink_from_to(0, "libEGL.so.1", X_DRV32_DIR, "libEGL.so.1", X_ETCLIB_SYML32_DIR);
		if( is_exists_dir_file(g.nv_drv32_dir, "libGLdispatch.so.0") )
		    mk_symlink32(0, "libGLdispatch.so.0", X_ETCLIB_SYML32_DIR);
		else
		    mk_symlink_from_to(0, "libGLdispatch.so.0", X_DRV32_DIR, "libGLdispatch.so.0", X_ETCLIB_SYML32_DIR);
		if( is_exists_dir_file(g.nv_drv32_dir, "libGLX.so.0") )
		    mk_symlink32(0, "libGLX.so.0", X_ETCLIB_SYML32_DIR);
		else
		    mk_symlink_from_to(0, "libGLX.so.0", X_DRV32_DIR, "libGLX.so.0", X_ETCLIB_SYML32_DIR);
	    }
	    /* restore symlinks from nvidia packages */
	    mk_symlink32(-1, "libGLcore.so.1", NV_ETCLIB_SYML32_DIR);
	    mk_symlink32(-1, "libEGL_nvidia.so.0", NV_ETCLIB_SYML32_DIR);
	    mk_symlink32(-1, "libGLESv2_nvidia.so.2", NV_ETCLIB_SYML32_DIR);
	    mk_symlink32(-1, "libGLESv1_CM_nvidia.so.1", NV_ETCLIB_SYML32_DIR);
	    mk_symlink32(-1, "libGLX_nvidia.so.0", NV_ETCLIB_SYML32_DIR);
	}
#endif

    /* switch tls lib */
    {
	mk_symlink_to(-1, "libnvidia-tls-tls.so.1", NV_ETCLIB_SYML_DIR, "libnvidia-tls.so.1");
#if defined(__x86_64__) || defined(__aarch64__)
	if( detected_libs_32() )
	    mk_symlink32_to(-1, "libnvidia-tls-tls.so.1", NV_ETCLIB_SYML32_DIR, "libnvidia-tls.so.1");
#endif
    }

#ifdef REMOVE_OLD_X11R6_LIBS
    /* remove old libraries */
    if( strcmp(XLIB_DIR, XLIB_DIR_OLD) != 0 )
    {
	unlink_file(XLIB_DIR_OLD, "libGL.so.1");
	unlink_file(XLIB_DIR_OLD, "libGLcore.so.1");
    }
#endif

    return(0);
}
